package de.unikoblenz.fgbks.base.utils.boundary.bicreater;

import static de.unikoblenz.fgbks.base.utils.boundary.BoundType.EXCLUSIVE;
import static de.unikoblenz.fgbks.base.utils.boundary.BoundType.INCLUSIVE;
import static de.unikoblenz.fgbks.base.utils.boundary.bicreater.BoundaryBiCreaterType.LOWER_BOUNDS;

import de.unikoblenz.fgbks.base.utils.boundary.AbstractGrowingBoundary;
import de.unikoblenz.fgbks.base.utils.boundary.impl.IntegerBoundary;
import de.unikoblenz.fgbks.base.utils.boundary.impl.LongBoundary;
import java.util.Optional;

/**
 * The lower bound creator implementation of {@link BoundaryBiCreaterType#LOWER_BOUNDS} for {@link
 * AbstractGrowingBoundary}. <br> The result is a new boundary with the lower bounds, or an empty
 * optional, if the creation is not possible. <br> In: {@code b1: [10..20]; b2: [15..30]} <br> Out:
 * {@code [10..15[} <br>
 */
public class BiCreaterLowerBounds extends AbstractBoundaryBiCreater<AbstractGrowingBoundary> {

  private static final BiCreaterLowerBounds instance = new BiCreaterLowerBounds();

  private BiCreaterLowerBounds() {
    super();
  }

  /**
   * Get the current instance of this class.
   *
   * @return the singleton instance
   */
  public static BiCreaterLowerBounds getInstance() {
    return instance;
  }

  @Override
  public BoundaryBiCreaterType getType() {
    return LOWER_BOUNDS;
  }

  @Override
  public Optional<AbstractGrowingBoundary> create(
      AbstractGrowingBoundary b1, AbstractGrowingBoundary b2) {
    if (b1 instanceof IntegerBoundary) {
      if ((int) b1.getLowerBound() + 1 == (int) b2.getLowerBound()
          && b1.getLowerBoundType() == EXCLUSIVE
          && b2.getLowerBoundType() == INCLUSIVE) {
        return Optional.empty();
      }
    }
    if (b1 instanceof LongBoundary) {
      if ((long) b1.getLowerBound() + 1 == (int) b2.getLowerBound()
          && b1.getLowerBoundType() == EXCLUSIVE
          && b2.getLowerBoundType() == INCLUSIVE) {
        return Optional.empty();
      }
    }
    if (b2.getLowerBound().equals(b2.getMinValue())) {
      return Optional.empty();
    }
    try {
      return Optional.of(
          b1.getCopy(
              b1.getLowerBound(),
              b2.getLowerBound(),
              b1.getLowerBoundType(),
              b2.getLowerBoundType().getOp()));
    } catch (Exception e) {
      return Optional.empty();
    }
  }
}
