package de.unikoblenz.fgbks.base.utils.boundary.bicreater;

import static de.unikoblenz.fgbks.base.utils.boundary.bicreater.BoundaryBiCreaterType.COMBINE;
import static de.unikoblenz.fgbks.base.utils.boundary.checker.BoundaryCheckType.IS_IN_CONTACT;

import de.unikoblenz.fgbks.base.utils.boundary.Boundary;
import de.unikoblenz.fgbks.base.utils.boundary.impl.StringBoundary;
import java.util.Optional;

/**
 * The between creator implementation of {@link BoundaryBiCreaterType#BETWEEN} for {@link
 * StringBoundary}. <br> The result is a new boundary with boundary between the both boundaries, or
 * an empty optional, if the creation is not possible. The boundaries are not in contact. <br> In:
 * {@code b1: "a"; b2: "b"} <br> Out: {@code not("a","b")} <br>
 */
public class BiCreaterStringAppend extends AbstractBoundaryBiCreater<StringBoundary> {

  private static final BiCreaterStringAppend instance = new BiCreaterStringAppend();

  private BiCreaterStringAppend() {
    super();
  }

  /**
   * Get the current instance of this class.
   *
   * @return the singleton instance
   */
  public static BiCreaterStringAppend getInstance() {
    return instance;
  }

  @Override
  public BoundaryBiCreaterType getType() {
    return BoundaryBiCreaterType.APPEND;
  }

  @Override
  public Optional<StringBoundary> create(StringBoundary b1, StringBoundary b2) {
    if (b1.checkWith(IS_IN_CONTACT, b2)) {
      return Optional.empty();
    }
    Optional<Boundary> b = b1.createBi(COMBINE, b2);
    return b.map(boundary -> (StringBoundary) boundary);
  }
}
