package de.unikoblenz.fgbks.base.utils.boundary.checker;

import de.unikoblenz.fgbks.base.utils.boundary.AbstractGrowingBoundary;

/**
 * The subsumes checker implementation of {@link BoundaryCheckType#IS_SUBSUMPTION} for {@link
 * AbstractGrowingBoundary}. <br> The check returns true, if the first boundary completely subsumes
 * (not equal) the other boundary or the other way around.
 * <br>
 * E.g: <br>
 *
 * <ul>
 *   <li>b1 = "[0..20]"; b2 = "[4..19]" -{@literal >} true
 *   <li>b1 = "[4..19]"; b2 = "[1..19]" -{@literal >} true
 *   <li>b1 = "[0..20]"; b2 = "[4..22]" -{@literal >} false
 * </ul>
 */
public class CheckIsSubsumption extends AbstractBoundaryCheck<AbstractGrowingBoundary> {

  private static final CheckIsSubsumption instance = new CheckIsSubsumption();

  private CheckIsSubsumption() {
    super();
  }

  /**
   * Get the current instance of this class.
   *
   * @return the singleton instance
   */
  public static CheckIsSubsumption getInstance() {
    return instance;
  }

  @Override
  public BoundaryCheckType getType() {
    return BoundaryCheckType.IS_SUBSUMPTION;
  }

  @Override
  public boolean check(AbstractGrowingBoundary b1, AbstractGrowingBoundary b2) {
    return checkIsSubsumption(b1, b2);
  }

  /**
   * see {@link CheckIsSubsumption}
   */
  public static boolean checkIsSubsumption(AbstractGrowingBoundary b1, AbstractGrowingBoundary b2) {
    return CheckSubsumes.checkSubsumes(b1, b2) || CheckSubsumes.checkSubsumes(b2, b1);
  }
}
